# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the dialogue classes of the tac participation skill."""

from pathlib import Path
from typing import cast

from aea.test_tools.test_skill import BaseSkillTestCase, COUNTERPARTY_AGENT_ADDRESS

from packages.fetchai.protocols.oef_search.message import OefSearchMessage
from packages.fetchai.protocols.state_update.message import StateUpdateMessage
from packages.fetchai.protocols.tac.message import TacMessage
from packages.fetchai.skills.tac_participation.dialogues import (
    OefSearchDialogue,
    OefSearchDialogues,
    StateUpdateDialogue,
    StateUpdateDialogues,
    TacDialogue,
    TacDialogues,
)

from tests.conftest import ROOT_DIR


class TestDialogues(BaseSkillTestCase):
    """Test dialogue classes of tac participation."""

    path_to_skill = Path(ROOT_DIR, "packages", "fetchai", "skills", "tac_participation")

    @classmethod
    def setup(cls):
        """Setup the test class."""
        super().setup()
        cls.oef_search_dialogues = cast(
            OefSearchDialogues, cls._skill.skill_context.oef_search_dialogues
        )
        cls.state_update_dialogues = cast(
            StateUpdateDialogues, cls._skill.skill_context.state_update_dialogues
        )
        cls.tac_dialogues = cast(TacDialogues, cls._skill.skill_context.tac_dialogues)

    def test_oef_search_dialogues(self):
        """Test the OefSearchDialogues class."""
        _, dialogue = self.oef_search_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=OefSearchMessage.Performative.SEARCH_SERVICES,
            query="some_query",
        )
        assert dialogue.role == OefSearchDialogue.Role.AGENT
        assert dialogue.self_address == str(self.skill.skill_context.skill_id)

    def test_state_update_dialogues(self):
        """Test the StateUpdateDialogues class."""
        _, dialogue = self.state_update_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=StateUpdateMessage.Performative.INITIALIZE,
            exchange_params_by_currency_id={"some_currency_id": 1.0},
            utility_params_by_good_id={"some_good_id": 2.0},
            amount_by_currency_id={"some_currency_id": 10},
            quantities_by_good_id={"some_good_id": 5},
        )
        assert dialogue.role == StateUpdateDialogue.Role.SKILL
        assert dialogue.self_address == str(self.skill.skill_context.skill_id)

    def test_tac_dialogues(self):
        """Test the TacDialogues class."""
        _, dialogue = self.tac_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=TacMessage.Performative.REGISTER,
            agent_name="some_agent_name",
        )
        assert dialogue.role == TacDialogue.Role.PARTICIPANT
        assert dialogue.self_address == self.skill.skill_context.agent_address



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains types and helpers for libp2p connections Uris."""

from random import randint
from typing import Optional


class Uri:
    """Holds a node address in format "host:port"."""

    __slots__ = ("_host", "_port")

    def __init__(
        self,
        uri: Optional[str] = None,
        host: Optional[str] = None,
        port: Optional[int] = None,
    ) -> None:
        """Initialise Uri."""
        if uri is not None:
            split = uri.split(":", 1)
            self._host = split[0]
            self._port = int(split[1])
        elif host is not None and port is not None:
            self._host = host
            self._port = port
        else:
            self._host = "127.0.0.1"
            self._port = randint(5000, 10000)  # nosec

    def __str__(self) -> str:
        """Get string representation."""
        return "{}:{}".format(self._host, self._port)

    def __repr__(self) -> str:  # pragma: no cover
        """Get object representation."""
        return self.__str__()

    @property
    def host(self) -> str:
        """Get host."""
        return self._host

    @property
    def port(self) -> int:
        """Get port."""
        return self._port



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests for the content of multi-agent-manager.md file."""
import os
import shutil
import tempfile
from importlib import import_module
from pathlib import Path
from unittest.mock import MagicMock

from tests.conftest import PACKAGES_DIR, ROOT_DIR
from tests.test_docs.helper import BasePythonMarkdownDocs


def _import_module_mock(arg):
    """
    Mock importlib.import_module only if argument is a dummy one: 'some.dotted.path'.

    This choice is tight to the code examples in 'ledger-integration.md'.
    It helps to tests the cases in which the import path is not a fake one.
    """
    if arg.startswith("some.dotted.path"):
        return MagicMock()
    return import_module(arg)


class TestMultiAgentManager(BasePythonMarkdownDocs):
    """Test the ledger integration code snippets."""

    DOC_PATH = Path(ROOT_DIR, "docs", "multi-agent-manager.md")

    @classmethod
    def setup_class(cls):
        """Set up the class."""
        super().setup_class()
        cls.old_cwd = os.getcwd()
        cls.temp_dir = tempfile.mkdtemp()
        shutil.copytree(PACKAGES_DIR, Path(cls.temp_dir, "packages"))
        os.chdir(cls.temp_dir)

    @classmethod
    def teardown_class(cls):
        """Teardown class."""
        os.chdir(cls.old_cwd)
        shutil.rmtree(cls.temp_dir)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains tests behaviour storage access."""
import os
from typing import List, Set

import pytest

from aea.aea import AEA
from aea.aea_builder import AEABuilder
from aea.configurations.base import SkillConfig
from aea.configurations.constants import DEFAULT_LEDGER
from aea.mail.base import Envelope
from aea.protocols.base import Address, Message
from aea.protocols.dialogue.base import (
    BasicDialoguesStorage,
    Dialogue,
    DialogueLabel,
    PersistDialoguesStorage,
)
from aea.skills.base import Handler, Skill, SkillContext
from aea.skills.behaviours import TickerBehaviour
from aea.test_tools.test_cases import AEATestCaseEmpty

from packages.fetchai.protocols.default.dialogues import (
    DefaultDialogue,
    DefaultDialogues,
)
from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.skills.echo import PUBLIC_ID

from tests.common.utils import wait_for_condition


class TBehaviour(TickerBehaviour):
    """Simple behaviour to count how many acts were called."""

    OBJ_ID = "some"
    OBJ_BODY = {"data": 12}
    COL_NAME = "test"

    def setup(self) -> None:
        """Set up behaviour."""
        self.counter = 0

    def act(self) -> None:
        """Make an action."""
        if self.context.storage and self.context.storage.is_connected:
            col = self.context.storage.get_sync_collection(self.COL_NAME)
            col.put(self.OBJ_ID, self.OBJ_BODY)
        self.counter += 1


class THandler(Handler):
    """Simple behaviour to count how many acts were called."""

    SUPPORTED_PROTOCOL = DefaultMessage.protocol_id

    OBJ_ID = "some"
    OBJ_BODY = {"data": 12}
    COL_NAME = "test"

    def setup(self) -> None:
        """Set up behaviour."""
        self.counter = 0

    def teardown(self) -> None:
        """Tear down handler."""

    def handle(self, *args, **kwargs) -> None:
        """Handle an evelope."""
        if self.context.storage and self.context.storage.is_connected:
            col = self.context.storage.get_sync_collection(self.COL_NAME)
            col.put(self.OBJ_ID, self.OBJ_BODY)

        self.counter += 1


def test_storage_access_from_behaviour():
    """Test storage access from behaviour component."""
    builder = AEABuilder()
    builder.set_name("aea_1")
    builder.add_private_key(DEFAULT_LEDGER)

    skill_context = SkillContext()
    behaviour = TBehaviour(name="behaviour", skill_context=skill_context)
    test_skill = Skill(
        SkillConfig(name="test_skill", author="fetchai"),
        skill_context=skill_context,
        handlers={},
        behaviours={"behaviour": behaviour},
    )

    builder.add_component_instance(test_skill)
    builder.set_storage_uri("sqlite://:memory:")
    aea = builder.build()
    skill_context.set_agent_context(aea.context)

    aea.runtime._threaded = True
    aea.runtime.start()

    try:
        wait_for_condition(lambda: aea.is_running, timeout=10)
        wait_for_condition(lambda: behaviour.counter > 0, timeout=10)

        col = skill_context.storage.get_sync_collection(behaviour.COL_NAME)
        assert col.get(behaviour.OBJ_ID) == behaviour.OBJ_BODY
    finally:
        aea.runtime.stop()
        aea.runtime.wait_completed(sync=True, timeout=10)


def test_storage_access_from_handler():
    """Test storage access from handler component."""
    builder = AEABuilder()
    builder.set_name("aea_1")
    builder.add_private_key(DEFAULT_LEDGER)

    skill_context = SkillContext()
    handler = THandler(name="behaviour", skill_context=skill_context)
    test_skill = Skill(
        SkillConfig(name="test_skill", author="fetchai"),
        skill_context=skill_context,
        handlers={"handler": handler},
        behaviours={},
    )

    builder.add_component_instance(test_skill)
    builder.set_storage_uri("sqlite://:memory:")
    aea = builder.build()
    skill_context.set_agent_context(aea.context)

    aea.runtime._threaded = True
    aea.runtime.start()

    msg = DefaultMessage(
        dialogue_reference=("", ""),
        message_id=1,
        target=0,
        performative=DefaultMessage.Performative.BYTES,
        content=b"hello",
    )
    msg.to = aea.identity.address
    msg.sender = aea.identity.address
    envelope = Envelope(to=msg.to, sender=msg.sender, message=msg,)
    try:
        wait_for_condition(lambda: aea.is_running, timeout=10)

        aea.runtime.multiplexer.in_queue.put(envelope)

        wait_for_condition(lambda: handler.counter > 0, timeout=10)

        col = skill_context.storage.get_sync_collection(handler.COL_NAME)
        assert col.get(handler.OBJ_ID) == handler.OBJ_BODY
    finally:
        aea.runtime.stop()
        aea.runtime.wait_completed(sync=True, timeout=10)


def _get_labels(dialogues: List[Dialogue]) -> Set[DialogueLabel]:
    return set([i.dialogue_label for i in dialogues])


def _storage_all_dialogues_labels(storage: BasicDialoguesStorage) -> Set[DialogueLabel]:
    return _get_labels(
        storage.dialogues_in_active_state + storage.dialogues_in_terminal_state
    )


class TestDialogueModelSaveLoad(AEATestCaseEmpty):
    """Test dialogues sved and loaded on agent restart."""

    def setup(self):
        """Set up the test case."""
        self.add_item("skill", "fetchai/echo:latest", local=True)
        pkey_file = os.path.join(self._get_cwd(), "privkey")
        self.generate_private_key("fetchai", pkey_file)
        self.add_private_key("fetchai", pkey_file, False)
        self.add_private_key("fetchai", pkey_file, True)

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> Dialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            return DefaultDialogue.Role.AGENT

        self.dialogues = DefaultDialogues(
            self_address="another_agent",
            role_from_first_message=role_from_first_message,
        )

    def _build_aea(self) -> AEA:
        """Build an AEA."""
        builder = AEABuilder.from_aea_project(self._get_cwd())
        builder.set_storage_uri("sqlite://some_file.db")
        aea = builder.build()
        aea.runtime._threaded = True
        return aea

    def test_dialogues_dumped_and_restored_properly(self):
        """Test dialogues restored during restart of agent."""
        aea = self._build_aea()
        aea.runtime.start()
        try:
            wait_for_condition(lambda: aea.is_running, timeout=10)
            echo_skill = aea.resources.get_skill(PUBLIC_ID)
            assert (
                not echo_skill.skill_context.default_dialogues._dialogues_storage._dialogues_by_dialogue_label
            )
            msg, dialogue = self.dialogues.create(
                aea.name,
                performative=DefaultMessage.Performative.BYTES,
                content=b"hello",
            )
            envelope = Envelope(to=msg.to, sender=msg.sender, message=msg,)
            aea.runtime.multiplexer.in_queue.put(envelope)

            dialogue_storage: PersistDialoguesStorage = echo_skill.skill_context.default_dialogues._dialogues_storage
            wait_for_condition(
                lambda: _storage_all_dialogues_labels(dialogue_storage), timeout=3,
            )
            dialogues_for_check = _storage_all_dialogues_labels(dialogue_storage)
        finally:
            aea.runtime.stop()
            aea.runtime.wait_completed(sync=True, timeout=10)

        aea = self._build_aea()
        aea.runtime.start()
        try:
            wait_for_condition(lambda: aea.is_running, timeout=10)
            echo_skill = aea.resources.get_skill(PUBLIC_ID)

            dialogue_storage: PersistDialoguesStorage = echo_skill.skill_context.default_dialogues._dialogues_storage
            wait_for_condition(
                lambda: _storage_all_dialogues_labels(dialogue_storage), timeout=3,
            )
            assert (
                _storage_all_dialogues_labels(dialogue_storage) == dialogues_for_check
            )
        finally:
            aea.runtime.stop()
            aea.runtime.wait_completed(sync=True, timeout=10)


if __name__ == "__main__":
    pytest.main([os.path.basename(__file__)])



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the class to connect to an Oracle contract."""

import logging
from typing import Any, Dict

from aea_ledger_ethereum import EthereumApi
from vyper.utils import keccak256

from aea.common import Address, JSONLike
from aea.configurations.base import PublicId
from aea.contracts.base import Contract
from aea.crypto.base import LedgerApi


PUBLIC_ID = PublicId.from_str("fetchai/oracle:0.5.0")
CONTRACT_ROLE = keccak256(b"ORACLE_ROLE")

_default_logger = logging.getLogger("aea.packages.fetchai.contracts.oracle.contract")


class FetchOracleContract(Contract):
    """The Fetch oracle contract."""

    @classmethod
    def get_grant_role_transaction(
        cls,
        ledger_api: LedgerApi,
        contract_address: Address,
        oracle_address: Address,
        gas: int = 0,
    ) -> JSONLike:
        """
        Get transaction to grant oracle role to recipient_address

        :param ledger_api: the ledger API
        :param contract_address: the contract address
        :param oracle_address: the address of the oracle
        :param gas: the gas limit
        :return: the transaction object
        """
        if ledger_api.identifier == EthereumApi.identifier:
            nonce = ledger_api.api.eth.getTransactionCount(oracle_address)
            instance = cls.get_instance(ledger_api, contract_address)
            tx = instance.functions.grantRole(
                CONTRACT_ROLE, oracle_address
            ).buildTransaction(
                {
                    "gas": gas,
                    "gasPrice": ledger_api.api.toWei("50", "gwei"),
                    "nonce": nonce,
                }
            )
            tx = ledger_api.update_with_gas_estimate(tx)
            return tx
        raise NotImplementedError

    @classmethod
    def get_update_transaction(
        cls,
        ledger_api: LedgerApi,
        contract_address: Address,
        oracle_address: Address,
        update_function: str,
        update_args: Dict[str, Any],
        gas: int = 0,
    ) -> JSONLike:
        """
        Update oracle value in contract

        :param ledger_api: the ledger apis.
        :param contract_address: the contract address.
        :param oracle_address: the oracle address.
        :param update_function: the oracle value update function.
        :param update_args: the arguments to the contract's update function.
        :return: None
        """
        if ledger_api.identifier == EthereumApi.identifier:
            nonce = ledger_api.api.eth.getTransactionCount(oracle_address)
            instance = cls.get_instance(ledger_api, contract_address)
            function = getattr(instance.functions, update_function)
            intermediate = function(*update_args)
            tx = intermediate.buildTransaction(
                {
                    "gas": gas,
                    "gasPrice": ledger_api.api.toWei("50", "gwei"),
                    "nonce": nonce,
                }
            )
            tx = ledger_api.update_with_gas_estimate(tx)
            return tx
        raise NotImplementedError



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This test module contains the tests for the configurations manager module."""
import os
from copy import deepcopy
from pathlib import Path
from unittest.mock import mock_open, patch

import pytest
import yaml

from aea.configurations.constants import CONNECTION
from aea.configurations.data_types import ComponentId, PublicId
from aea.configurations.manager import (
    AgentConfigManager,
    find_component_directory_from_component_id,
    handle_dotted_path,
)
from aea.configurations.validation import SAME_MARK
from aea.exceptions import AEAException

from tests.conftest import ROOT_DIR


DUMMY_AEA = Path(ROOT_DIR) / "tests" / "data" / "dummy_aea"

agent_config_data = yaml.safe_load(
    """
agent_name: Agent0
author: dummy_author
version: 1.0.0
description: dummy_aea agent description
license: Apache-2.0
aea_version: '>=0.11.0, <0.12.0'
fingerprint: {}
fingerprint_ignore_patterns: []
connections:
- fetchai/local:0.16.0
contracts: []
protocols:
- fetchai/default:0.13.0
skills:
- dummy_author/dummy:0.1.0
- fetchai/error:0.13.0
default_connection: fetchai/local:0.16.0
default_ledger: cosmos
logging_config:
  disable_existing_loggers: ${DISABLE_LOGS:bool}
  version: 1
private_key_paths:
  cosmos: cosmos_private_key.txt
  ethereum: ethereum_private_key.txt
connection_private_key_paths:
  cosmos: cosmos_private_key.txt
  ethereum: ethereum_private_key.txt
registry_path: ../../packages
default_routing: {}
"""
)


def test_envvars_applied():
    """Test env vars replaced with values."""
    dct = deepcopy(agent_config_data)
    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "true"
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=True)
    assert (
        agent_config_manager.json["logging_config"]["disable_existing_loggers"] is True
    )

    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "false"
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=True)
    assert (
        agent_config_manager.json["logging_config"]["disable_existing_loggers"] is False
    )

    # no env! no default value
    os.environ.pop("DISABLE_LOGS")
    with pytest.raises(
        ValueError,
        match="`DISABLE_LOGS` not found in env variables and no default value set!",
    ):
        with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
            agent_config_manager = AgentConfigManager.load(
                ".", substitude_env_vars=True
            )
        assert (
            agent_config_manager.json["logging_config"]["disable_existing_loggers"]
            is False
        )

    # check default value specified
    dct["logging_config"]["disable_existing_loggers"] = "${DISABLE_LOGS:bool:true}"
    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=True)
        assert (
            agent_config_manager.json["logging_config"]["disable_existing_loggers"]
            is True
        )

    # check incorrect data type
    dct["logging_config"]["disable_existing_loggers"] = "${DISABLE_LOGS:int:true}"
    with pytest.raises(ValueError, match="Cannot convert string `true` to type `int`"):
        with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
            agent_config_manager = AgentConfigManager.load(
                ".", substitude_env_vars=True
            )
            assert (
                agent_config_manager.json["logging_config"]["disable_existing_loggers"]
                is True
            )

    # not applied
    dct = deepcopy(agent_config_data)
    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "true"
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=False)
    assert (
        agent_config_manager.json["logging_config"]["disable_existing_loggers"]
        == dct["logging_config"]["disable_existing_loggers"]
    )


@patch.object(AgentConfigManager, "get_overridables", return_value=[{}, {}])
def test_envvars_preserved(*mocks):
    """Test env vars not modified on config update."""
    dct = deepcopy(agent_config_data)
    new_cosmos_key_value = "cosmons_key_updated"

    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "true"
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=False)

    assert (
        agent_config_manager.json["logging_config"]["disable_existing_loggers"]
        == dct["logging_config"]["disable_existing_loggers"]
    )

    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "true"
        agent_config_manager = AgentConfigManager.load(".", substitude_env_vars=False)

    assert (
        agent_config_manager.json["private_key_paths"]["cosmos"] != new_cosmos_key_value
    )
    agent_config_manager.update_config(
        {"private_key_paths": {"cosmos": new_cosmos_key_value}}
    )

    assert (
        agent_config_manager.json["logging_config"]["disable_existing_loggers"]
        == dct["logging_config"]["disable_existing_loggers"]
    )
    assert (
        agent_config_manager.json["private_key_paths"]["cosmos"] == new_cosmos_key_value
    )


def test_agent_attribute_get_set():
    """Test agent config manager  get set variables."""
    dct = deepcopy(agent_config_data)
    with patch.object(AgentConfigManager, "_load_config_data", return_value=[dct]):
        os.environ["DISABLE_LOGS"] = "true"
        agent_config_manager = AgentConfigManager.load(
            DUMMY_AEA, substitude_env_vars=False
        )
        assert (
            agent_config_manager.get_variable("agent.default_ledger")
            == dct["default_ledger"]
        )
        assert (
            agent_config_manager.get_variable("vendor.fetchai.skills.error.name")
            == "error"
        )

        assert (
            agent_config_manager.get_variable(
                "vendor.fetchai.connections.local.is_abstract"
            )
            is False
        )
        agent_config_manager.set_variable(
            "vendor.fetchai.connections.local.is_abstract", True
        )
        assert (
            agent_config_manager.get_variable(
                "vendor.fetchai.connections.local.is_abstract"
            )
            is True
        )

        agent_config_manager.set_variable("agent.default_ledger", "fetchai")
        assert agent_config_manager.get_variable("agent.default_ledger") == "fetchai"

        assert (
            agent_config_manager.json["component_configurations"][0]["is_abstract"]
            is True
        )

    agent_config_manager = AgentConfigManager.load(DUMMY_AEA, substitude_env_vars=False)
    agent_config_manager.set_variable(
        "vendor.fetchai.connections.p2p_libp2p.config.delegate_uri", "some_url"
    )
    assert (
        agent_config_manager.get_variable(
            "vendor.fetchai.connections.p2p_libp2p.config.delegate_uri"
        )
        == "some_url"
    )

    with pytest.raises(
        ValueError, match="Attribute `does_not_exist` for AgentConfig does not exist"
    ):
        agent_config_manager.get_variable("agent.does_not_exist")

    agent_config_manager.validate_current_config()
    agent_config_manager.verify_or_create_private_keys(DUMMY_AEA, lambda x, y, z: None)


def test_agent_attribute_get_overridables():
    """Test AgentConfigManager.get_overridables."""
    agent_config_manager = AgentConfigManager.load(DUMMY_AEA, substitude_env_vars=False)
    agent_overrides, component_overrides = agent_config_manager.get_overridables()
    assert "default_ledger" in agent_overrides
    assert "is_abstract" in list(component_overrides.values())[0]


def test_dump_config():
    """Test AgentConfigManager.dump_config."""
    agent_config_manager = AgentConfigManager.load(DUMMY_AEA, substitude_env_vars=False)
    with patch("aea.configurations.manager.open_file", mock_open()), patch(
        "aea.configurations.loader.ConfigLoader.dump"
    ) as dump_mock:
        agent_config_manager.dump_config()

    dump_mock.assert_called_once()


def test_handle_dotted_path():
    """Test handle_dotted_path."""
    with pytest.raises(
        AEAException, match=r"The root of the dotted path must be one of:"
    ):
        handle_dotted_path("something", author="fetchai")

    with pytest.raises(
        AEAException,
        match=r"The path is too short. Please specify a path up to an attribute name.",
    ):
        handle_dotted_path("skills", author="fetchai")

    with pytest.raises(
        AEAException, match=r"is not a valid component type. Please use one of"
    ):
        handle_dotted_path("vendor.fetchai.notskills.dummy.name", author="fetchai")

    with pytest.raises(AEAException, match=r"Resource .* does not exist."):
        handle_dotted_path("skills.notdummy.name", author="fetchai")


def test_find_component_directory_from_component_id():
    """Test find_component_directory_from_component_id."""
    with pytest.raises(ValueError, match=r"Package .* not found."):
        find_component_directory_from_component_id(
            Path("."),
            ComponentId(
                component_type=CONNECTION, public_id=PublicId("test", "test", "1.0.1")
            ),
        )


def test_agent_attribute_get_and_apply_overridables():
    """Test AgentConfigManager.get_overridables and apply it."""
    agent_config_manager = AgentConfigManager.load(DUMMY_AEA, substitude_env_vars=False)
    initial_agent_config_json = agent_config_manager.json

    agent_overrides, component_overrides = agent_config_manager.get_overridables()

    agent_config_manager.update_config(agent_overrides)
    assert initial_agent_config_json == agent_config_manager.json

    agent_overrides["component_configurations"] = component_overrides
    agent_config_manager.update_config(agent_overrides)

    assert agent_config_manager._filter_overrides(agent_overrides) == SAME_MARK

    agent_overrides["execution_timeout"] = 12
    assert agent_config_manager._filter_overrides(agent_overrides) == {
        "execution_timeout": 12
    }



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests for the content of language-agnostic-definition.md file."""
from pathlib import Path
from typing import Dict

from aea import AEA_DIR
from aea.configurations.data_types import PublicId

from tests.conftest import ROOT_DIR
from tests.test_docs.helper import BaseTestMarkdownDocs


MAIL_BASE_PROTO = Path(AEA_DIR) / "mail" / "base.proto"
DEFAULT_MESSAGE_PROTO = (
    Path(ROOT_DIR) / "packages" / "fetchai" / "protocols" / "default" / "default.proto"
)


class TestLanguageAgnosticDocs(BaseTestMarkdownDocs):
    """Test the integrity of the language agnostic definitions in docs."""

    DOC_PATH = Path(ROOT_DIR, "docs", "language-agnostic-definition.md")

    @classmethod
    def _proto_snippet_selector(cls, block: Dict) -> bool:
        return block["type"] == "block_code" and block["info"].strip() == "proto"

    @classmethod
    def setup_class(cls):
        """Set up the test."""
        super().setup_class()
        cls.code_blocks = list(filter(cls._proto_snippet_selector, cls.blocks))
        cls.actual_mail_base_file_content = MAIL_BASE_PROTO.read_text()
        cls.actual_default_message_file_content = DEFAULT_MESSAGE_PROTO.read_text()

    def test_envelope_code_snippet(self):
        """
        Test the Envelope protobuf code snippet.

        It requires treating the preamble lines separately,
        because aea/mail/base.proto doesn't follow
        the same order of language-agnostic-definition.md file.
        """
        block = self.code_blocks[0]
        assert block["info"].strip() == "proto"

        lines = block["text"].splitlines()
        first_part, second_part = "\n".join(lines[:3]), "\n".join(lines[3:])

        assert first_part in self.actual_mail_base_file_content
        assert second_part in self.actual_mail_base_file_content

    def test_dialogue_message_code_snippet(self):
        """
        Test the DialogueMessage protobuf code snippet.

        It requires treating the preamble lines separately,
        because aea/mail/base.proto doesn't follow
        the same order of language-agnostic-definition.md file.
        """
        block = self.code_blocks[1]
        assert block["info"].strip() == "proto"
        assert block["text"] in self.actual_mail_base_file_content

    def test_public_id_regular_expression(self):
        """Test public id regular expression is the same."""
        expected_regex = PublicId.PUBLIC_ID_REGEX
        assert expected_regex in self.doc_content

    def test_default_message_code_snippet(self):
        """Test DefaultMessage protobuf code snippet."""
        block = self.code_blocks[2]
        assert block["info"].strip() == "proto"
        assert block["text"] in self.actual_default_message_file_content, Exception(
            [block["text"], self.actual_default_message_file_content]
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests of the oef_search protocol package."""

import sys
from typing import Type
from unittest import mock

import pytest

from aea.common import Address
from aea.exceptions import AEAEnforceError
from aea.helpers.search.models import Constraint, ConstraintType, Description, Query
from aea.mail.base import Envelope
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue as BaseDialogue
from aea.protocols.dialogue.base import DialogueLabel

import packages
from packages.fetchai.protocols.oef_search.dialogues import (
    OefSearchDialogue,
    OefSearchDialogues,
)
from packages.fetchai.protocols.oef_search.message import OefSearchMessage
from packages.fetchai.protocols.oef_search.message import (
    _default_logger as oef_search_message_logger,
)

from tests.conftest import ROOT_DIR


sys.path.append(ROOT_DIR)


def test_register_service_serialization():
    """Test the serialization for 'register_service' speech-act works."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.REGISTER_SERVICE,
        service_description=Description({"foo1": 1, "bar1": 2}),
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_unregister_service_serialization():
    """Test the serialization for 'unregister_service' speech-act works."""
    msg = OefSearchMessage(
        message_id=2,
        target=1,
        performative=OefSearchMessage.Performative.UNREGISTER_SERVICE,
        service_description=Description({"foo1": 1, "bar1": 2}),
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_search_services_serialization():
    """Test the serialization for 'search_services' speech-act works."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.SEARCH_SERVICES,
        query=Query([Constraint("something", ConstraintType(">", 1))]),
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_search_result_serialization():
    """Test the serialization for 'search_result' speech-act works."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.SEARCH_RESULT,
        agents=("agent_1", "agent_2", "agent_3"),
        agents_info=OefSearchMessage.AgentsInfo(
            {
                "key_1": {"key_1": b"value_1", "key_2": b"value_2"},
                "key_2": {"key_3": b"value_3", "key_4": b"value_4"},
            }
        ),
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_success_serialization():
    """Test the serialization for 'success' speech-act works."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.SUCCESS,
        agents_info=OefSearchMessage.AgentsInfo(
            {
                "key_1": {"key_1": b"value_1", "key_2": b"value_2"},
                "key_2": {"key_3": b"value_3", "key_4": b"value_4"},
            }
        ),
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_oef_error_serialization():
    """Test the serialization for 'oef_error' speech-act works."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.OEF_ERROR,
        oef_error_operation=OefSearchMessage.OefErrorOperation.OTHER,
    )
    msg.to = "receiver"
    envelope = Envelope(to=msg.to, sender="sender", message=msg,)
    envelope_bytes = envelope.encode()

    actual_envelope = Envelope.decode(envelope_bytes)
    expected_envelope = envelope
    assert expected_envelope.to == actual_envelope.to
    assert expected_envelope.sender == actual_envelope.sender
    assert (
        expected_envelope.protocol_specification_id
        == actual_envelope.protocol_specification_id
    )
    assert expected_envelope.message != actual_envelope.message

    actual_msg = OefSearchMessage.serializer.decode(actual_envelope.message)
    actual_msg.to = actual_envelope.to
    actual_msg.sender = actual_envelope.sender
    expected_msg = msg
    assert expected_msg == actual_msg


def test_oef_type_string_value():
    """Test the string value of the type."""
    assert (
        str(OefSearchMessage.Performative.REGISTER_SERVICE) == "register_service"
    ), "The str value must be register_service"
    assert (
        str(OefSearchMessage.Performative.UNREGISTER_SERVICE) == "unregister_service"
    ), "The str value must be unregister_service"
    assert (
        str(OefSearchMessage.Performative.SEARCH_SERVICES) == "search_services"
    ), "The str value must be search_services"
    assert (
        str(OefSearchMessage.Performative.OEF_ERROR) == "oef_error"
    ), "The str value must be oef_error"
    assert (
        str(OefSearchMessage.Performative.SEARCH_RESULT) == "search_result"
    ), "The str value must be search_result"


def test_oef_error_operation():
    """Test the string value of the error operation."""
    assert (
        str(OefSearchMessage.OefErrorOperation.REGISTER_SERVICE) == "0"
    ), "The str value must be 0"
    assert (
        str(OefSearchMessage.OefErrorOperation.UNREGISTER_SERVICE) == "1"
    ), "The str value must be 1"
    assert (
        str(OefSearchMessage.OefErrorOperation.SEARCH_SERVICES) == "2"
    ), "The str value must be 2"
    assert (
        str(OefSearchMessage.OefErrorOperation.SEND_MESSAGE) == "3"
    ), "The str value must be 3"
    assert (
        str(OefSearchMessage.OefErrorOperation.OTHER) == "10000"
    ), "The str value must be 10000"


def test_encoding_unknown_performative():
    """Test that we raise an exception when the performative is unknown during encoding."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.REGISTER_SERVICE,
        service_description=Description({"foo1": 1, "bar1": 2}),
    )

    with pytest.raises(ValueError, match="Performative not valid:"):
        with mock.patch.object(
            OefSearchMessage.Performative, "__eq__", return_value=False
        ):
            OefSearchMessage.serializer.encode(msg)


def test_decoding_unknown_performative():
    """Test that we raise an exception when the performative is unknown during decoding."""
    msg = OefSearchMessage(
        performative=OefSearchMessage.Performative.REGISTER_SERVICE,
        service_description=Description({"foo1": 1, "bar1": 2}),
    )

    encoded_msg = OefSearchMessage.serializer.encode(msg)
    with pytest.raises(ValueError, match="Performative not valid:"):
        with mock.patch.object(
            OefSearchMessage.Performative, "__eq__", return_value=False
        ):
            OefSearchMessage.serializer.decode(encoded_msg)


@mock.patch.object(
    packages.fetchai.protocols.oef_search.message,
    "enforce",
    side_effect=AEAEnforceError("some error"),
)
def test_incorrect_message(mocked_enforce):
    """Test that we raise an exception when the fipa message is incorrect."""
    with mock.patch.object(oef_search_message_logger, "error") as mock_logger:
        OefSearchMessage(
            performative=OefSearchMessage.Performative.REGISTER_SERVICE,
            service_description=Description({"foo1": 1, "bar1": 2}),
        )

        mock_logger.assert_any_call("some error")


def test_agent_info():
    """Test the agent_info custom type."""
    agents_info = OefSearchMessage.AgentsInfo(
        {
            "agent_address_1": {"key_1": b"value_1", "key_2": b"value_2"},
            "agent_address_2": {"key_3": b"value_3", "key_4": b"value_4"},
        }
    )
    assert agents_info.get_info_for_agent("agent_address_1") == {
        "key_1": b"value_1",
        "key_2": b"value_2",
    }

    with pytest.raises(ValueError, match="body must not be None"):
        OefSearchMessage.AgentsInfo(None)


class TestDialogues:
    """Tests oef_search dialogues."""

    @classmethod
    def setup_class(cls):
        """Set up the test."""
        cls.agent_addr = "agent address"
        cls.oef_node_addr = "oef_node address"
        cls.agent_dialogues = BuyerDialogues(cls.agent_addr)
        cls.oef_node_dialogues = OEFNodeDialogues(cls.oef_node_addr)

    def test_create_self_initiated(self):
        """Test the self initialisation of a dialogue."""
        result = self.agent_dialogues._create_self_initiated(
            dialogue_opponent_addr=self.oef_node_addr,
            dialogue_reference=(str(0), ""),
            role=OefSearchDialogue.Role.AGENT,
        )
        assert isinstance(result, OefSearchDialogue)
        assert result.role == OefSearchDialogue.Role.AGENT, "The role must be agent."

    def test_create_opponent_initiated(self):
        """Test the opponent initialisation of a dialogue."""
        result = self.agent_dialogues._create_opponent_initiated(
            dialogue_opponent_addr=self.oef_node_addr,
            dialogue_reference=(str(0), ""),
            role=OefSearchDialogue.Role.AGENT,
        )
        assert isinstance(result, OefSearchDialogue)
        assert result.role == OefSearchDialogue.Role.AGENT, "The role must be agent."


class BuyerDialogue(OefSearchDialogue):
    """The dialogue class maintains state of a dialogue and manages it."""

    def __init__(
        self,
        dialogue_label: DialogueLabel,
        self_address: Address,
        role: BaseDialogue.Role,
        message_class: Type[OefSearchMessage],
    ) -> None:
        """
        Initialize a dialogue.

        :param dialogue_label: the identifier of the dialogue
        :param self_address: the address of the entity for whom this dialogue is maintained
        :param role: the role of the agent this dialogue is maintained for

        :return: None
        """
        OefSearchDialogue.__init__(
            self,
            dialogue_label=dialogue_label,
            self_address=self_address,
            role=role,
            message_class=message_class,
        )


class BuyerDialogues(OefSearchDialogues):
    """The dialogues class keeps track of all dialogues."""

    def __init__(self, self_address: Address) -> None:
        """
        Initialize dialogues.

        :return: None
        """

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> BaseDialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            return OefSearchDialogue.Role.AGENT

        OefSearchDialogues.__init__(
            self,
            self_address=self_address,
            role_from_first_message=role_from_first_message,
            dialogue_class=BuyerDialogue,
        )


class OEFNodeDialogue(OefSearchDialogue):
    """The dialogue class maintains state of a dialogue and manages it."""

    def __init__(
        self,
        dialogue_label: DialogueLabel,
        self_address: Address,
        role: BaseDialogue.Role,
        message_class: Type[OefSearchMessage],
    ) -> None:
        """
        Initialize a dialogue.

        :param dialogue_label: the identifier of the dialogue
        :param self_address: the address of the entity for whom this dialogue is maintained
        :param role: the role of the agent this dialogue is maintained for

        :return: None
        """
        OefSearchDialogue.__init__(
            self,
            dialogue_label=dialogue_label,
            self_address=self_address,
            role=role,
            message_class=message_class,
        )


class OEFNodeDialogues(OefSearchDialogues):
    """The dialogues class keeps track of all dialogues."""

    def __init__(self, self_address: Address) -> None:
        """
        Initialize dialogues.

        :return: None
        """

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> BaseDialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            return OefSearchDialogue.Role.OEF_NODE

        OefSearchDialogues.__init__(
            self,
            self_address=self_address,
            role_from_first_message=role_from_first_message,
            dialogue_class=OEFNodeDialogue,
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the yoti connection module."""
import asyncio
import logging
from typing import cast
from unittest.mock import MagicMock, Mock, patch
from uuid import uuid4

import pytest

from aea.mail.base import Envelope
from aea.protocols.base import Address, Message
from aea.protocols.dialogue.base import Dialogue

from packages.fetchai.connections.yoti import connection as yoti_connection
from packages.fetchai.connections.yoti.connection import YotiConnection
from packages.fetchai.protocols.yoti.dialogues import YotiDialogue
from packages.fetchai.protocols.yoti.dialogues import YotiDialogues as BaseYotiDialogues
from packages.fetchai.protocols.yoti.message import YotiMessage


SKILL_ID = "some/skill:0.1.0"


class FakeYotiClientOk:
    """Test yoti client."""

    def __init__(self, *args, **kwargs):
        """Init client."""

    def callable(self, *args):
        """Some callable."""
        m = Mock()
        m.sources = []
        m.verifiers = []
        return m

    def get_activity_details(self, *args):
        """Get activity details."""
        activity_details = Mock()
        activity_details.profile.attributes = {}
        activity_details.profile.a = self.callable
        return activity_details


class FakeYotiClientBadProfile(FakeYotiClientOk):
    """Get a bad profile."""

    def get_activity_details(self, *args):
        """Get activity details."""
        activity_details = Mock()
        activity_details.profile = None
        return activity_details


class YotiDialogues(BaseYotiDialogues):
    """The dialogues class keeps track of all dialogues."""

    def __init__(self, **kwargs) -> None:
        """
        Initialize dialogues.

        :return: None
        """

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> Dialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            # The yoti connection maintains the dialogue on behalf of the yoti server
            return YotiDialogue.Role.AGENT

        BaseYotiDialogues.__init__(
            self,
            self_address=SKILL_ID,
            role_from_first_message=role_from_first_message,
            **kwargs,
        )


mock_conf = Mock()
mock_conf.public_id = YotiConnection.connection_id
mock_conf.config = {"yoti_client_sdk_id": "1", "yoti_key_file_path": 1}


@pytest.mark.asyncio
async def test_yoti_profile_ok():
    """Test mesasge processed ok."""
    with patch.object(yoti_connection, "YotiClient", FakeYotiClientOk):
        con = YotiConnection(
            configuration=mock_conf, data_dir=MagicMock(), logger=logging.getLogger()
        )
        await con.connect()
        assert con.is_connected

        dialogues = YotiDialogues()

        message, _ = dialogues.create(
            str(con.connection_id),
            performative=YotiMessage.Performative.GET_PROFILE,
            token=str(uuid4()),
            dotted_path="",
        )
        envelope = Envelope(to=str(con.connection_id), sender=SKILL_ID, message=message)
        await con.send(envelope)
        resp_envelope = await asyncio.wait_for(con.receive(), timeout=1)
        assert (
            cast(YotiMessage, resp_envelope.message.performative)
            == YotiMessage.Performative.PROFILE
        )

        message, _ = dialogues.create(
            str(con.connection_id),
            performative=YotiMessage.Performative.GET_PROFILE,
            token=str(uuid4()),
            dotted_path="a",
            args=("1",),
        )
        envelope = Envelope(to=str(con.connection_id), sender=SKILL_ID, message=message)
        await con.send(envelope)
        resp_envelope = await asyncio.wait_for(con.receive(), timeout=1)
        assert (
            cast(YotiMessage, resp_envelope.message.performative)
            == YotiMessage.Performative.PROFILE
        )

        await con.disconnect()
        assert con.is_disconnected


@pytest.mark.asyncio
async def test_yoti_profile_error_on_handle():
    """Test error message on bad provile."""
    with patch.object(
        yoti_connection, "YotiClient", FakeYotiClientBadProfile,
    ):
        con = YotiConnection(
            configuration=mock_conf, data_dir=MagicMock(), logger=logging.getLogger()
        )
        await con.connect()
        dialogues = YotiDialogues()

        message, _ = dialogues.create(
            str(con.connection_id),
            performative=YotiMessage.Performative.GET_PROFILE,
            token=str(uuid4()),
            dotted_path="",
        )
        envelope = Envelope(to=str(con.connection_id), sender=SKILL_ID, message=message)
        await con.send(envelope)
        resp_envelope = await asyncio.wait_for(con.receive(), timeout=1)
        assert (
            cast(YotiMessage, resp_envelope.message.performative)
            == YotiMessage.Performative.ERROR
        )
        await con.disconnect()


def test_empty_config():
    """Test no sdk or token provided."""
    mock_conf = Mock()
    mock_conf.public_id = YotiConnection.connection_id
    mock_conf.config = {}
    with pytest.raises(ValueError, match="Missing configuration."):
        YotiConnection(configuration=mock_conf, data_dir=MagicMock())


class TestMisc:
    """Misc tests."""

    def setup(self):
        """Setup test case."""
        with patch.object(yoti_connection, "YotiClient", FakeYotiClientOk):
            self.con = YotiConnection(
                configuration=mock_conf,
                data_dir=MagicMock(),
                logger=logging.getLogger(),
            )
        self.dialogues = YotiDialogues()
        self.message, self.dialogue = self.dialogues.create(
            str(self.con.connection_id),
            performative=YotiMessage.Performative.GET_PROFILE,
            token=str(uuid4()),
            dotted_path="a",
            args=tuple(),
        )
        self.envelope = Envelope(
            to=str(self.con.connection_id), sender=SKILL_ID, message=self.message
        )

    def test_yoti_no_response_from_handler(self):
        """Test yoti handler returns no response."""
        with patch.object(self.con, "get_profile", return_value=None):
            self.con.dispatch(self.envelope)

    def test_no_handler(self):
        """Test bad peroformative for get handler."""
        with pytest.raises(Exception, match="Performative not recognized."):
            self.con.get_handler("some")

    def test_get_no_profile_data(self):
        """Test no profile data returned."""
        with patch.object(self.con._client, "get_activity_details", return_value=None):
            resp = self.con.get_profile(self.message, self.dialogue)

        assert resp
        assert resp.error_code == 500
        assert resp.error_msg == "No activity_details returned"

    def test_get_attribute(self):
        """Test get not callable attribute of profile."""
        activity_details = Mock()
        activity_details.profile.attributes = {}
        activity_details.profile.a.sources = []
        activity_details.profile.a.verifiers = []

        with patch.object(
            self.con._client, "get_activity_details", return_value=activity_details
        ):
            resp = self.con.get_profile(self.message, self.dialogue)

        assert resp
        assert resp.performative == YotiMessage.Performative.PROFILE



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This test module contains the tests for the scaffold connection."""
import os
import sys
from unittest.mock import MagicMock

import pytest

from aea.configurations.base import ConnectionConfig
from aea.connections.scaffold.connection import MyScaffoldAsyncConnection
from aea.helpers.base import cd
from aea.test_tools.test_cases import AEATestCaseEmpty

from tests.common.pexpect_popen import PexpectWrapper
from tests.conftest import ROOT_DIR


class TestScaffoldConnectionReception:
    """Test that the stub connection is implemented correctly."""

    def setup(self):
        """Set case up."""
        configuration = ConnectionConfig(
            connection_id=MyScaffoldAsyncConnection.connection_id,
        )
        self.connection = MyScaffoldAsyncConnection(
            configuration=configuration, data_dir=MagicMock()
        )

    @pytest.mark.asyncio
    async def test_methods_not_implemented(self):
        """Test methods not implemented."""
        with pytest.raises(NotImplementedError):
            await self.connection.connect()

        with pytest.raises(NotImplementedError):
            await self.connection.disconnect()

        with pytest.raises(NotImplementedError):
            await self.connection.send(None)

        with pytest.raises(NotImplementedError):
            await self.connection.receive()


class TestScaffoldConnectionAndRun(AEATestCaseEmpty):
    """Test that the scaffold connection created."""

    def setup(self):
        """Set case up."""
        result = self.invoke("scaffold", "connection", "my_con")
        assert result.exit_code == 0

    def test_run_and_not_implemented_error(self):
        """Test aea run crashes with connect not implemented for scaffolded connection."""
        with cd(self._get_cwd()):
            proc = PexpectWrapper(  # nosec
                [sys.executable, "-m", "aea.cli", "-v", "DEBUG", "run"],
                env={
                    **os.environ,
                    "PYTHONPATH": ROOT_DIR + ":" + os.environ.get("PYTHONPATH", ""),
                },
                maxread=10000,
                encoding="utf-8",
                logfile=sys.stdout,
            )
            try:
                proc.expect_all(
                    [
                        "Error while connecting <class 'connection_module.MyScaffoldAsyncConnection'>: NotImplementedError()"
                    ],
                    timeout=50,
                )
            finally:
                proc.terminate()
                proc.wait_to_complete(timeout=50)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: models.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="models.proto",
    package="aea.helpers.search.models",
    syntax="proto3",
    serialized_options=b"H\001",
    serialized_pb=b'\n\x0cmodels.proto\x12\x19\x61\x65\x61.helpers.search.models"\xc1\x19\n\x05Query\x1a\xc0\x01\n\tAttribute\x12\x0c\n\x04name\x18\x01 \x01(\t\x12=\n\x04type\x18\x02 \x01(\x0e\x32/.aea.helpers.search.models.Query.Attribute.Type\x12\x10\n\x08required\x18\x03 \x01(\x08\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t"?\n\x04Type\x12\n\n\x06\x44OUBLE\x10\x00\x12\x07\n\x03INT\x10\x01\x12\x08\n\x04\x42OOL\x10\x02\x12\n\n\x06STRING\x10\x03\x12\x0c\n\x08LOCATION\x10\x04\x1an\n\tDataModel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12>\n\nattributes\x18\x02 \x03(\x0b\x32*.aea.helpers.search.models.Query.Attribute\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a$\n\x08Location\x12\x0b\n\x03lon\x18\x01 \x01(\x01\x12\x0b\n\x03lat\x18\x02 \x01(\x01\x1a\x99\x01\n\x05Value\x12\x10\n\x06string\x18\x01 \x01(\tH\x00\x12\x10\n\x06\x64ouble\x18\x02 \x01(\x01H\x00\x12\x11\n\x07\x62oolean\x18\x03 \x01(\x08H\x00\x12\x11\n\x07integer\x18\x04 \x01(\x03H\x00\x12=\n\x08location\x18\x05 \x01(\x0b\x32).aea.helpers.search.models.Query.LocationH\x00\x42\x07\n\x05value\x1aN\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.aea.helpers.search.models.Query.Value\x1a\x80\x01\n\x08Instance\x12\x39\n\x05model\x18\x01 \x01(\x0b\x32*.aea.helpers.search.models.Query.DataModel\x12\x39\n\x06values\x18\x02 \x03(\x0b\x32).aea.helpers.search.models.Query.KeyValue\x1a+\n\nStringPair\x12\r\n\x05\x66irst\x18\x01 \x01(\t\x12\x0e\n\x06second\x18\x02 \x01(\t\x1a(\n\x07IntPair\x12\r\n\x05\x66irst\x18\x01 \x01(\x03\x12\x0e\n\x06second\x18\x02 \x01(\x03\x1a+\n\nDoublePair\x12\r\n\x05\x66irst\x18\x01 \x01(\x01\x12\x0e\n\x06second\x18\x02 \x01(\x01\x1a\x83\x01\n\x0cLocationPair\x12\x38\n\x05\x66irst\x18\x01 \x01(\x0b\x32).aea.helpers.search.models.Query.Location\x12\x39\n\x06second\x18\x02 \x01(\x0b\x32).aea.helpers.search.models.Query.Location\x1a\xa1\x02\n\x05Range\x12\x42\n\x0bstring_pair\x18\x01 \x01(\x0b\x32+.aea.helpers.search.models.Query.StringPairH\x00\x12@\n\x0cinteger_pair\x18\x02 \x01(\x0b\x32(.aea.helpers.search.models.Query.IntPairH\x00\x12\x42\n\x0b\x64ouble_pair\x18\x03 \x01(\x0b\x32+.aea.helpers.search.models.Query.DoublePairH\x00\x12\x46\n\rlocation_pair\x18\x04 \x01(\x0b\x32-.aea.helpers.search.models.Query.LocationPairH\x00\x42\x06\n\x04pair\x1aW\n\x08\x44istance\x12\x39\n\x06\x63\x65nter\x18\x01 \x01(\x0b\x32).aea.helpers.search.models.Query.Location\x12\x10\n\x08\x64istance\x18\x02 \x01(\x01\x1a\xca\x01\n\x08Relation\x12\x44\n\x08operator\x18\x01 \x01(\x0e\x32\x32.aea.helpers.search.models.Query.Relation.Operator\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.aea.helpers.search.models.Query.Value"A\n\x08Operator\x12\x06\n\x02\x45Q\x10\x00\x12\x06\n\x02LT\x10\x01\x12\x08\n\x04LTEQ\x10\x02\x12\x06\n\x02GT\x10\x03\x12\x08\n\x04GTEQ\x10\x04\x12\t\n\x05NOTEQ\x10\x05\x1a\xca\x05\n\x03Set\x12?\n\x08operator\x18\x01 \x01(\x0e\x32-.aea.helpers.search.models.Query.Set.Operator\x12;\n\x06values\x18\x02 \x01(\x0b\x32+.aea.helpers.search.models.Query.Set.Values\x1a\xa5\x04\n\x06Values\x12\x45\n\x06string\x18\x01 \x01(\x0b\x32\x33.aea.helpers.search.models.Query.Set.Values.StringsH\x00\x12\x45\n\x06\x64ouble\x18\x02 \x01(\x0b\x32\x33.aea.helpers.search.models.Query.Set.Values.DoublesH\x00\x12\x44\n\x07\x62oolean\x18\x03 \x01(\x0b\x32\x31.aea.helpers.search.models.Query.Set.Values.BoolsH\x00\x12\x43\n\x07integer\x18\x04 \x01(\x0b\x32\x30.aea.helpers.search.models.Query.Set.Values.IntsH\x00\x12I\n\x08location\x18\x05 \x01(\x0b\x32\x35.aea.helpers.search.models.Query.Set.Values.LocationsH\x00\x1a\x16\n\x04Ints\x12\x0e\n\x06values\x18\x01 \x03(\x03\x1a\x19\n\x07\x44oubles\x12\x0e\n\x06values\x18\x01 \x03(\x01\x1a\x19\n\x07Strings\x12\x0e\n\x06values\x18\x01 \x03(\t\x1a\x17\n\x05\x42ools\x12\x0e\n\x06values\x18\x01 \x03(\x08\x1a\x46\n\tLocations\x12\x39\n\x06values\x18\x01 \x03(\x0b\x32).aea.helpers.search.models.Query.LocationB\x08\n\x06values"\x1d\n\x08Operator\x12\x06\n\x02IN\x10\x00\x12\t\n\x05NOTIN\x10\x01\x1a\xc3\x06\n\x0e\x43onstraintExpr\x12\x41\n\x03or_\x18\x01 \x01(\x0b\x32\x32.aea.helpers.search.models.Query.ConstraintExpr.OrH\x00\x12\x43\n\x04\x61nd_\x18\x02 \x01(\x0b\x32\x33.aea.helpers.search.models.Query.ConstraintExpr.AndH\x00\x12\x43\n\x04not_\x18\x03 \x01(\x0b\x32\x33.aea.helpers.search.models.Query.ConstraintExpr.NotH\x00\x12P\n\nconstraint\x18\x04 \x01(\x0b\x32:.aea.helpers.search.models.Query.ConstraintExpr.ConstraintH\x00\x1aI\n\x02Or\x12\x43\n\nexpression\x18\x01 \x03(\x0b\x32/.aea.helpers.search.models.Query.ConstraintExpr\x1aJ\n\x03\x41nd\x12\x43\n\nexpression\x18\x01 \x03(\x0b\x32/.aea.helpers.search.models.Query.ConstraintExpr\x1aJ\n\x03Not\x12\x43\n\nexpression\x18\x01 \x01(\x0b\x32/.aea.helpers.search.models.Query.ConstraintExpr\x1a\xa0\x02\n\nConstraint\x12\x16\n\x0e\x61ttribute_name\x18\x01 \x01(\t\x12\x34\n\x04set_\x18\x02 \x01(\x0b\x32$.aea.helpers.search.models.Query.SetH\x00\x12\x38\n\x06range_\x18\x03 \x01(\x0b\x32&.aea.helpers.search.models.Query.RangeH\x00\x12=\n\x08relation\x18\x04 \x01(\x0b\x32).aea.helpers.search.models.Query.RelationH\x00\x12=\n\x08\x64istance\x18\x05 \x01(\x0b\x32).aea.helpers.search.models.Query.DistanceH\x00\x42\x0c\n\nconstraintB\x0c\n\nexpression\x1a\x88\x01\n\x05Model\x12\x44\n\x0b\x63onstraints\x18\x01 \x03(\x0b\x32/.aea.helpers.search.models.Query.ConstraintExpr\x12\x39\n\x05model\x18\x02 \x01(\x0b\x32*.aea.helpers.search.models.Query.DataModelB\x02H\x01\x62\x06proto3',
)


_QUERY_ATTRIBUTE_TYPE = _descriptor.EnumDescriptor(
    name="Type",
    full_name="aea.helpers.search.models.Query.Attribute.Type",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="DOUBLE", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="INT", index=1, number=1, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="BOOL", index=2, number=2, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="STRING", index=3, number=3, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="LOCATION", index=4, number=4, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=183,
    serialized_end=246,
)
_sym_db.RegisterEnumDescriptor(_QUERY_ATTRIBUTE_TYPE)

_QUERY_RELATION_OPERATOR = _descriptor.EnumDescriptor(
    name="Operator",
    full_name="aea.helpers.search.models.Query.Relation.Operator",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="EQ", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="LT", index=1, number=1, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="LTEQ", index=2, number=2, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="GT", index=3, number=3, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="GTEQ", index=4, number=4, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="NOTEQ", index=5, number=5, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1550,
    serialized_end=1615,
)
_sym_db.RegisterEnumDescriptor(_QUERY_RELATION_OPERATOR)

_QUERY_SET_OPERATOR = _descriptor.EnumDescriptor(
    name="Operator",
    full_name="aea.helpers.search.models.Query.Set.Operator",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="IN", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="NOTIN", index=1, number=1, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2303,
    serialized_end=2332,
)
_sym_db.RegisterEnumDescriptor(_QUERY_SET_OPERATOR)


_QUERY_ATTRIBUTE = _descriptor.Descriptor(
    name="Attribute",
    full_name="aea.helpers.search.models.Query.Attribute",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="aea.helpers.search.models.Query.Attribute.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="type",
            full_name="aea.helpers.search.models.Query.Attribute.type",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="required",
            full_name="aea.helpers.search.models.Query.Attribute.required",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="aea.helpers.search.models.Query.Attribute.description",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[_QUERY_ATTRIBUTE_TYPE,],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=54,
    serialized_end=246,
)

_QUERY_DATAMODEL = _descriptor.Descriptor(
    name="DataModel",
    full_name="aea.helpers.search.models.Query.DataModel",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="aea.helpers.search.models.Query.DataModel.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="attributes",
            full_name="aea.helpers.search.models.Query.DataModel.attributes",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="aea.helpers.search.models.Query.DataModel.description",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=248,
    serialized_end=358,
)

_QUERY_LOCATION = _descriptor.Descriptor(
    name="Location",
    full_name="aea.helpers.search.models.Query.Location",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="lon",
            full_name="aea.helpers.search.models.Query.Location.lon",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="lat",
            full_name="aea.helpers.search.models.Query.Location.lat",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=360,
    serialized_end=396,
)

_QUERY_VALUE = _descriptor.Descriptor(
    name="Value",
    full_name="aea.helpers.search.models.Query.Value",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="string",
            full_name="aea.helpers.search.models.Query.Value.string",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="double",
            full_name="aea.helpers.search.models.Query.Value.double",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="boolean",
            full_name="aea.helpers.search.models.Query.Value.boolean",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="integer",
            full_name="aea.helpers.search.models.Query.Value.integer",
            index=3,
            number=4,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="location",
            full_name="aea.helpers.search.models.Query.Value.location",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="value",
            full_name="aea.helpers.search.models.Query.Value.value",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=399,
    serialized_end=552,
)

_QUERY_KEYVALUE = _descriptor.Descriptor(
    name="KeyValue",
    full_name="aea.helpers.search.models.Query.KeyValue",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="aea.helpers.search.models.Query.KeyValue.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.helpers.search.models.Query.KeyValue.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=554,
    serialized_end=632,
)

_QUERY_INSTANCE = _descriptor.Descriptor(
    name="Instance",
    full_name="aea.helpers.search.models.Query.Instance",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="model",
            full_name="aea.helpers.search.models.Query.Instance.model",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Instance.values",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=635,
    serialized_end=763,
)

_QUERY_STRINGPAIR = _descriptor.Descriptor(
    name="StringPair",
    full_name="aea.helpers.search.models.Query.StringPair",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="first",
            full_name="aea.helpers.search.models.Query.StringPair.first",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="second",
            full_name="aea.helpers.search.models.Query.StringPair.second",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=765,
    serialized_end=808,
)

_QUERY_INTPAIR = _descriptor.Descriptor(
    name="IntPair",
    full_name="aea.helpers.search.models.Query.IntPair",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="first",
            full_name="aea.helpers.search.models.Query.IntPair.first",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="second",
            full_name="aea.helpers.search.models.Query.IntPair.second",
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=810,
    serialized_end=850,
)

_QUERY_DOUBLEPAIR = _descriptor.Descriptor(
    name="DoublePair",
    full_name="aea.helpers.search.models.Query.DoublePair",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="first",
            full_name="aea.helpers.search.models.Query.DoublePair.first",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="second",
            full_name="aea.helpers.search.models.Query.DoublePair.second",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=852,
    serialized_end=895,
)

_QUERY_LOCATIONPAIR = _descriptor.Descriptor(
    name="LocationPair",
    full_name="aea.helpers.search.models.Query.LocationPair",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="first",
            full_name="aea.helpers.search.models.Query.LocationPair.first",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="second",
            full_name="aea.helpers.search.models.Query.LocationPair.second",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=898,
    serialized_end=1029,
)

_QUERY_RANGE = _descriptor.Descriptor(
    name="Range",
    full_name="aea.helpers.search.models.Query.Range",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="string_pair",
            full_name="aea.helpers.search.models.Query.Range.string_pair",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="integer_pair",
            full_name="aea.helpers.search.models.Query.Range.integer_pair",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="double_pair",
            full_name="aea.helpers.search.models.Query.Range.double_pair",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="location_pair",
            full_name="aea.helpers.search.models.Query.Range.location_pair",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="pair",
            full_name="aea.helpers.search.models.Query.Range.pair",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=1032,
    serialized_end=1321,
)

_QUERY_DISTANCE = _descriptor.Descriptor(
    name="Distance",
    full_name="aea.helpers.search.models.Query.Distance",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="center",
            full_name="aea.helpers.search.models.Query.Distance.center",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="distance",
            full_name="aea.helpers.search.models.Query.Distance.distance",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1323,
    serialized_end=1410,
)

_QUERY_RELATION = _descriptor.Descriptor(
    name="Relation",
    full_name="aea.helpers.search.models.Query.Relation",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="operator",
            full_name="aea.helpers.search.models.Query.Relation.operator",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="aea.helpers.search.models.Query.Relation.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[_QUERY_RELATION_OPERATOR,],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1413,
    serialized_end=1615,
)

_QUERY_SET_VALUES_INTS = _descriptor.Descriptor(
    name="Ints",
    full_name="aea.helpers.search.models.Query.Set.Values.Ints",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.Ints.values",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2118,
    serialized_end=2140,
)

_QUERY_SET_VALUES_DOUBLES = _descriptor.Descriptor(
    name="Doubles",
    full_name="aea.helpers.search.models.Query.Set.Values.Doubles",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.Doubles.values",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2142,
    serialized_end=2167,
)

_QUERY_SET_VALUES_STRINGS = _descriptor.Descriptor(
    name="Strings",
    full_name="aea.helpers.search.models.Query.Set.Values.Strings",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.Strings.values",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2169,
    serialized_end=2194,
)

_QUERY_SET_VALUES_BOOLS = _descriptor.Descriptor(
    name="Bools",
    full_name="aea.helpers.search.models.Query.Set.Values.Bools",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.Bools.values",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2196,
    serialized_end=2219,
)

_QUERY_SET_VALUES_LOCATIONS = _descriptor.Descriptor(
    name="Locations",
    full_name="aea.helpers.search.models.Query.Set.Values.Locations",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.Locations.values",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2221,
    serialized_end=2291,
)

_QUERY_SET_VALUES = _descriptor.Descriptor(
    name="Values",
    full_name="aea.helpers.search.models.Query.Set.Values",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="string",
            full_name="aea.helpers.search.models.Query.Set.Values.string",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="double",
            full_name="aea.helpers.search.models.Query.Set.Values.double",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="boolean",
            full_name="aea.helpers.search.models.Query.Set.Values.boolean",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="integer",
            full_name="aea.helpers.search.models.Query.Set.Values.integer",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="location",
            full_name="aea.helpers.search.models.Query.Set.Values.location",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _QUERY_SET_VALUES_INTS,
        _QUERY_SET_VALUES_DOUBLES,
        _QUERY_SET_VALUES_STRINGS,
        _QUERY_SET_VALUES_BOOLS,
        _QUERY_SET_VALUES_LOCATIONS,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.Values.values",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=1752,
    serialized_end=2301,
)

_QUERY_SET = _descriptor.Descriptor(
    name="Set",
    full_name="aea.helpers.search.models.Query.Set",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="operator",
            full_name="aea.helpers.search.models.Query.Set.operator",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="values",
            full_name="aea.helpers.search.models.Query.Set.values",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[_QUERY_SET_VALUES,],
    enum_types=[_QUERY_SET_OPERATOR,],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1618,
    serialized_end=2332,
)

_QUERY_CONSTRAINTEXPR_OR = _descriptor.Descriptor(
    name="Or",
    full_name="aea.helpers.search.models.Query.ConstraintExpr.Or",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="expression",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Or.expression",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2640,
    serialized_end=2713,
)

_QUERY_CONSTRAINTEXPR_AND = _descriptor.Descriptor(
    name="And",
    full_name="aea.helpers.search.models.Query.ConstraintExpr.And",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="expression",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.And.expression",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2715,
    serialized_end=2789,
)

_QUERY_CONSTRAINTEXPR_NOT = _descriptor.Descriptor(
    name="Not",
    full_name="aea.helpers.search.models.Query.ConstraintExpr.Not",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="expression",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Not.expression",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2791,
    serialized_end=2865,
)

_QUERY_CONSTRAINTEXPR_CONSTRAINT = _descriptor.Descriptor(
    name="Constraint",
    full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="attribute_name",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.attribute_name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="set_",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.set_",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="range_",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.range_",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="relation",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.relation",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="distance",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.distance",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="constraint",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.Constraint.constraint",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=2868,
    serialized_end=3156,
)

_QUERY_CONSTRAINTEXPR = _descriptor.Descriptor(
    name="ConstraintExpr",
    full_name="aea.helpers.search.models.Query.ConstraintExpr",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="or_",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.or_",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="and_",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.and_",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="not_",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.not_",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="constraint",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.constraint",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _QUERY_CONSTRAINTEXPR_OR,
        _QUERY_CONSTRAINTEXPR_AND,
        _QUERY_CONSTRAINTEXPR_NOT,
        _QUERY_CONSTRAINTEXPR_CONSTRAINT,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="expression",
            full_name="aea.helpers.search.models.Query.ConstraintExpr.expression",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=2335,
    serialized_end=3170,
)

_QUERY_MODEL = _descriptor.Descriptor(
    name="Model",
    full_name="aea.helpers.search.models.Query.Model",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="constraints",
            full_name="aea.helpers.search.models.Query.Model.constraints",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="model",
            full_name="aea.helpers.search.models.Query.Model.model",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3173,
    serialized_end=3309,
)

_QUERY = _descriptor.Descriptor(
    name="Query",
    full_name="aea.helpers.search.models.Query",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[
        _QUERY_ATTRIBUTE,
        _QUERY_DATAMODEL,
        _QUERY_LOCATION,
        _QUERY_VALUE,
        _QUERY_KEYVALUE,
        _QUERY_INSTANCE,
        _QUERY_STRINGPAIR,
        _QUERY_INTPAIR,
        _QUERY_DOUBLEPAIR,
        _QUERY_LOCATIONPAIR,
        _QUERY_RANGE,
        _QUERY_DISTANCE,
        _QUERY_RELATION,
        _QUERY_SET,
        _QUERY_CONSTRAINTEXPR,
        _QUERY_MODEL,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=44,
    serialized_end=3309,
)

_QUERY_ATTRIBUTE.fields_by_name["type"].enum_type = _QUERY_ATTRIBUTE_TYPE
_QUERY_ATTRIBUTE.containing_type = _QUERY
_QUERY_ATTRIBUTE_TYPE.containing_type = _QUERY_ATTRIBUTE
_QUERY_DATAMODEL.fields_by_name["attributes"].message_type = _QUERY_ATTRIBUTE
_QUERY_DATAMODEL.containing_type = _QUERY
_QUERY_LOCATION.containing_type = _QUERY
_QUERY_VALUE.fields_by_name["location"].message_type = _QUERY_LOCATION
_QUERY_VALUE.containing_type = _QUERY
_QUERY_VALUE.oneofs_by_name["value"].fields.append(
    _QUERY_VALUE.fields_by_name["string"]
)
_QUERY_VALUE.fields_by_name["string"].containing_oneof = _QUERY_VALUE.oneofs_by_name[
    "value"
]
_QUERY_VALUE.oneofs_by_name["value"].fields.append(
    _QUERY_VALUE.fields_by_name["double"]
)
_QUERY_VALUE.fields_by_name["double"].containing_oneof = _QUERY_VALUE.oneofs_by_name[
    "value"
]
_QUERY_VALUE.oneofs_by_name["value"].fields.append(
    _QUERY_VALUE.fields_by_name["boolean"]
)
_QUERY_VALUE.fields_by_name["boolean"].containing_oneof = _QUERY_VALUE.oneofs_by_name[
    "value"
]
_QUERY_VALUE.oneofs_by_name["value"].fields.append(
    _QUERY_VALUE.fields_by_name["integer"]
)
_QUERY_VALUE.fields_by_name["integer"].containing_oneof = _QUERY_VALUE.oneofs_by_name[
    "value"
]
_QUERY_VALUE.oneofs_by_name["value"].fields.append(
    _QUERY_VALUE.fields_by_name["location"]
)
_QUERY_VALUE.fields_by_name["location"].containing_oneof = _QUERY_VALUE.oneofs_by_name[
    "value"
]
_QUERY_KEYVALUE.fields_by_name["value"].message_type = _QUERY_VALUE
_QUERY_KEYVALUE.containing_type = _QUERY
_QUERY_INSTANCE.fields_by_name["model"].message_type = _QUERY_DATAMODEL
_QUERY_INSTANCE.fields_by_name["values"].message_type = _QUERY_KEYVALUE
_QUERY_INSTANCE.containing_type = _QUERY
_QUERY_STRINGPAIR.containing_type = _QUERY
_QUERY_INTPAIR.containing_type = _QUERY
_QUERY_DOUBLEPAIR.containing_type = _QUERY
_QUERY_LOCATIONPAIR.fields_by_name["first"].message_type = _QUERY_LOCATION
_QUERY_LOCATIONPAIR.fields_by_name["second"].message_type = _QUERY_LOCATION
_QUERY_LOCATIONPAIR.containing_type = _QUERY
_QUERY_RANGE.fields_by_name["string_pair"].message_type = _QUERY_STRINGPAIR
_QUERY_RANGE.fields_by_name["integer_pair"].message_type = _QUERY_INTPAIR
_QUERY_RANGE.fields_by_name["double_pair"].message_type = _QUERY_DOUBLEPAIR
_QUERY_RANGE.fields_by_name["location_pair"].message_type = _QUERY_LOCATIONPAIR
_QUERY_RANGE.containing_type = _QUERY
_QUERY_RANGE.oneofs_by_name["pair"].fields.append(
    _QUERY_RANGE.fields_by_name["string_pair"]
)
_QUERY_RANGE.fields_by_name[
    "string_pair"
].containing_oneof = _QUERY_RANGE.oneofs_by_name["pair"]
_QUERY_RANGE.oneofs_by_name["pair"].fields.append(
    _QUERY_RANGE.fields_by_name["integer_pair"]
)
_QUERY_RANGE.fields_by_name[
    "integer_pair"
].containing_oneof = _QUERY_RANGE.oneofs_by_name["pair"]
_QUERY_RANGE.oneofs_by_name["pair"].fields.append(
    _QUERY_RANGE.fields_by_name["double_pair"]
)
_QUERY_RANGE.fields_by_name[
    "double_pair"
].containing_oneof = _QUERY_RANGE.oneofs_by_name["pair"]
_QUERY_RANGE.oneofs_by_name["pair"].fields.append(
    _QUERY_RANGE.fields_by_name["location_pair"]
)
_QUERY_RANGE.fields_by_name[
    "location_pair"
].containing_oneof = _QUERY_RANGE.oneofs_by_name["pair"]
_QUERY_DISTANCE.fields_by_name["center"].message_type = _QUERY_LOCATION
_QUERY_DISTANCE.containing_type = _QUERY
_QUERY_RELATION.fields_by_name["operator"].enum_type = _QUERY_RELATION_OPERATOR
_QUERY_RELATION.fields_by_name["value"].message_type = _QUERY_VALUE
_QUERY_RELATION.containing_type = _QUERY
_QUERY_RELATION_OPERATOR.containing_type = _QUERY_RELATION
_QUERY_SET_VALUES_INTS.containing_type = _QUERY_SET_VALUES
_QUERY_SET_VALUES_DOUBLES.containing_type = _QUERY_SET_VALUES
_QUERY_SET_VALUES_STRINGS.containing_type = _QUERY_SET_VALUES
_QUERY_SET_VALUES_BOOLS.containing_type = _QUERY_SET_VALUES
_QUERY_SET_VALUES_LOCATIONS.fields_by_name["values"].message_type = _QUERY_LOCATION
_QUERY_SET_VALUES_LOCATIONS.containing_type = _QUERY_SET_VALUES
_QUERY_SET_VALUES.fields_by_name["string"].message_type = _QUERY_SET_VALUES_STRINGS
_QUERY_SET_VALUES.fields_by_name["double"].message_type = _QUERY_SET_VALUES_DOUBLES
_QUERY_SET_VALUES.fields_by_name["boolean"].message_type = _QUERY_SET_VALUES_BOOLS
_QUERY_SET_VALUES.fields_by_name["integer"].message_type = _QUERY_SET_VALUES_INTS
_QUERY_SET_VALUES.fields_by_name["location"].message_type = _QUERY_SET_VALUES_LOCATIONS
_QUERY_SET_VALUES.containing_type = _QUERY_SET
_QUERY_SET_VALUES.oneofs_by_name["values"].fields.append(
    _QUERY_SET_VALUES.fields_by_name["string"]
)
_QUERY_SET_VALUES.fields_by_name[
    "string"
].containing_oneof = _QUERY_SET_VALUES.oneofs_by_name["values"]
_QUERY_SET_VALUES.oneofs_by_name["values"].fields.append(
    _QUERY_SET_VALUES.fields_by_name["double"]
)
_QUERY_SET_VALUES.fields_by_name[
    "double"
].containing_oneof = _QUERY_SET_VALUES.oneofs_by_name["values"]
_QUERY_SET_VALUES.oneofs_by_name["values"].fields.append(
    _QUERY_SET_VALUES.fields_by_name["boolean"]
)
_QUERY_SET_VALUES.fields_by_name[
    "boolean"
].containing_oneof = _QUERY_SET_VALUES.oneofs_by_name["values"]
_QUERY_SET_VALUES.oneofs_by_name["values"].fields.append(
    _QUERY_SET_VALUES.fields_by_name["integer"]
)
_QUERY_SET_VALUES.fields_by_name[
    "integer"
].containing_oneof = _QUERY_SET_VALUES.oneofs_by_name["values"]
_QUERY_SET_VALUES.oneofs_by_name["values"].fields.append(
    _QUERY_SET_VALUES.fields_by_name["location"]
)
_QUERY_SET_VALUES.fields_by_name[
    "location"
].containing_oneof = _QUERY_SET_VALUES.oneofs_by_name["values"]
_QUERY_SET.fields_by_name["operator"].enum_type = _QUERY_SET_OPERATOR
_QUERY_SET.fields_by_name["values"].message_type = _QUERY_SET_VALUES
_QUERY_SET.containing_type = _QUERY
_QUERY_SET_OPERATOR.containing_type = _QUERY_SET
_QUERY_CONSTRAINTEXPR_OR.fields_by_name[
    "expression"
].message_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_OR.containing_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_AND.fields_by_name[
    "expression"
].message_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_AND.containing_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_NOT.fields_by_name[
    "expression"
].message_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_NOT.containing_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["set_"].message_type = _QUERY_SET
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["range_"].message_type = _QUERY_RANGE
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "relation"
].message_type = _QUERY_RELATION
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "distance"
].message_type = _QUERY_DISTANCE
_QUERY_CONSTRAINTEXPR_CONSTRAINT.containing_type = _QUERY_CONSTRAINTEXPR
_QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"].fields.append(
    _QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["set_"]
)
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "set_"
].containing_oneof = _QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"]
_QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"].fields.append(
    _QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["range_"]
)
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "range_"
].containing_oneof = _QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"]
_QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"].fields.append(
    _QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["relation"]
)
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "relation"
].containing_oneof = _QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"]
_QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"].fields.append(
    _QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name["distance"]
)
_QUERY_CONSTRAINTEXPR_CONSTRAINT.fields_by_name[
    "distance"
].containing_oneof = _QUERY_CONSTRAINTEXPR_CONSTRAINT.oneofs_by_name["constraint"]
_QUERY_CONSTRAINTEXPR.fields_by_name["or_"].message_type = _QUERY_CONSTRAINTEXPR_OR
_QUERY_CONSTRAINTEXPR.fields_by_name["and_"].message_type = _QUERY_CONSTRAINTEXPR_AND
_QUERY_CONSTRAINTEXPR.fields_by_name["not_"].message_type = _QUERY_CONSTRAINTEXPR_NOT
_QUERY_CONSTRAINTEXPR.fields_by_name[
    "constraint"
].message_type = _QUERY_CONSTRAINTEXPR_CONSTRAINT
_QUERY_CONSTRAINTEXPR.containing_type = _QUERY
_QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"].fields.append(
    _QUERY_CONSTRAINTEXPR.fields_by_name["or_"]
)
_QUERY_CONSTRAINTEXPR.fields_by_name[
    "or_"
].containing_oneof = _QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"]
_QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"].fields.append(
    _QUERY_CONSTRAINTEXPR.fields_by_name["and_"]
)
_QUERY_CONSTRAINTEXPR.fields_by_name[
    "and_"
].containing_oneof = _QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"]
_QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"].fields.append(
    _QUERY_CONSTRAINTEXPR.fields_by_name["not_"]
)
_QUERY_CONSTRAINTEXPR.fields_by_name[
    "not_"
].containing_oneof = _QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"]
_QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"].fields.append(
    _QUERY_CONSTRAINTEXPR.fields_by_name["constraint"]
)
_QUERY_CONSTRAINTEXPR.fields_by_name[
    "constraint"
].containing_oneof = _QUERY_CONSTRAINTEXPR.oneofs_by_name["expression"]
_QUERY_MODEL.fields_by_name["constraints"].message_type = _QUERY_CONSTRAINTEXPR
_QUERY_MODEL.fields_by_name["model"].message_type = _QUERY_DATAMODEL
_QUERY_MODEL.containing_type = _QUERY
DESCRIPTOR.message_types_by_name["Query"] = _QUERY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Query = _reflection.GeneratedProtocolMessageType(
    "Query",
    (_message.Message,),
    {
        "Attribute": _reflection.GeneratedProtocolMessageType(
            "Attribute",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_ATTRIBUTE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Attribute)
            },
        ),
        "DataModel": _reflection.GeneratedProtocolMessageType(
            "DataModel",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_DATAMODEL,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.DataModel)
            },
        ),
        "Location": _reflection.GeneratedProtocolMessageType(
            "Location",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_LOCATION,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Location)
            },
        ),
        "Value": _reflection.GeneratedProtocolMessageType(
            "Value",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_VALUE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Value)
            },
        ),
        "KeyValue": _reflection.GeneratedProtocolMessageType(
            "KeyValue",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_KEYVALUE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.KeyValue)
            },
        ),
        "Instance": _reflection.GeneratedProtocolMessageType(
            "Instance",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_INSTANCE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Instance)
            },
        ),
        "StringPair": _reflection.GeneratedProtocolMessageType(
            "StringPair",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_STRINGPAIR,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.StringPair)
            },
        ),
        "IntPair": _reflection.GeneratedProtocolMessageType(
            "IntPair",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_INTPAIR,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.IntPair)
            },
        ),
        "DoublePair": _reflection.GeneratedProtocolMessageType(
            "DoublePair",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_DOUBLEPAIR,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.DoublePair)
            },
        ),
        "LocationPair": _reflection.GeneratedProtocolMessageType(
            "LocationPair",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_LOCATIONPAIR,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.LocationPair)
            },
        ),
        "Range": _reflection.GeneratedProtocolMessageType(
            "Range",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_RANGE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Range)
            },
        ),
        "Distance": _reflection.GeneratedProtocolMessageType(
            "Distance",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_DISTANCE,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Distance)
            },
        ),
        "Relation": _reflection.GeneratedProtocolMessageType(
            "Relation",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_RELATION,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Relation)
            },
        ),
        "Set": _reflection.GeneratedProtocolMessageType(
            "Set",
            (_message.Message,),
            {
                "Values": _reflection.GeneratedProtocolMessageType(
                    "Values",
                    (_message.Message,),
                    {
                        "Ints": _reflection.GeneratedProtocolMessageType(
                            "Ints",
                            (_message.Message,),
                            {
                                "DESCRIPTOR": _QUERY_SET_VALUES_INTS,
                                "__module__": "models_pb2"
                                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values.Ints)
                            },
                        ),
                        "Doubles": _reflection.GeneratedProtocolMessageType(
                            "Doubles",
                            (_message.Message,),
                            {
                                "DESCRIPTOR": _QUERY_SET_VALUES_DOUBLES,
                                "__module__": "models_pb2"
                                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values.Doubles)
                            },
                        ),
                        "Strings": _reflection.GeneratedProtocolMessageType(
                            "Strings",
                            (_message.Message,),
                            {
                                "DESCRIPTOR": _QUERY_SET_VALUES_STRINGS,
                                "__module__": "models_pb2"
                                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values.Strings)
                            },
                        ),
                        "Bools": _reflection.GeneratedProtocolMessageType(
                            "Bools",
                            (_message.Message,),
                            {
                                "DESCRIPTOR": _QUERY_SET_VALUES_BOOLS,
                                "__module__": "models_pb2"
                                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values.Bools)
                            },
                        ),
                        "Locations": _reflection.GeneratedProtocolMessageType(
                            "Locations",
                            (_message.Message,),
                            {
                                "DESCRIPTOR": _QUERY_SET_VALUES_LOCATIONS,
                                "__module__": "models_pb2"
                                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values.Locations)
                            },
                        ),
                        "DESCRIPTOR": _QUERY_SET_VALUES,
                        "__module__": "models_pb2"
                        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set.Values)
                    },
                ),
                "DESCRIPTOR": _QUERY_SET,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Set)
            },
        ),
        "ConstraintExpr": _reflection.GeneratedProtocolMessageType(
            "ConstraintExpr",
            (_message.Message,),
            {
                "Or": _reflection.GeneratedProtocolMessageType(
                    "Or",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _QUERY_CONSTRAINTEXPR_OR,
                        "__module__": "models_pb2"
                        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.ConstraintExpr.Or)
                    },
                ),
                "And": _reflection.GeneratedProtocolMessageType(
                    "And",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _QUERY_CONSTRAINTEXPR_AND,
                        "__module__": "models_pb2"
                        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.ConstraintExpr.And)
                    },
                ),
                "Not": _reflection.GeneratedProtocolMessageType(
                    "Not",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _QUERY_CONSTRAINTEXPR_NOT,
                        "__module__": "models_pb2"
                        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.ConstraintExpr.Not)
                    },
                ),
                "Constraint": _reflection.GeneratedProtocolMessageType(
                    "Constraint",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _QUERY_CONSTRAINTEXPR_CONSTRAINT,
                        "__module__": "models_pb2"
                        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.ConstraintExpr.Constraint)
                    },
                ),
                "DESCRIPTOR": _QUERY_CONSTRAINTEXPR,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.ConstraintExpr)
            },
        ),
        "Model": _reflection.GeneratedProtocolMessageType(
            "Model",
            (_message.Message,),
            {
                "DESCRIPTOR": _QUERY_MODEL,
                "__module__": "models_pb2"
                # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query.Model)
            },
        ),
        "DESCRIPTOR": _QUERY,
        "__module__": "models_pb2"
        # @@protoc_insertion_point(class_scope:aea.helpers.search.models.Query)
    },
)
_sym_db.RegisterMessage(Query)
_sym_db.RegisterMessage(Query.Attribute)
_sym_db.RegisterMessage(Query.DataModel)
_sym_db.RegisterMessage(Query.Location)
_sym_db.RegisterMessage(Query.Value)
_sym_db.RegisterMessage(Query.KeyValue)
_sym_db.RegisterMessage(Query.Instance)
_sym_db.RegisterMessage(Query.StringPair)
_sym_db.RegisterMessage(Query.IntPair)
_sym_db.RegisterMessage(Query.DoublePair)
_sym_db.RegisterMessage(Query.LocationPair)
_sym_db.RegisterMessage(Query.Range)
_sym_db.RegisterMessage(Query.Distance)
_sym_db.RegisterMessage(Query.Relation)
_sym_db.RegisterMessage(Query.Set)
_sym_db.RegisterMessage(Query.Set.Values)
_sym_db.RegisterMessage(Query.Set.Values.Ints)
_sym_db.RegisterMessage(Query.Set.Values.Doubles)
_sym_db.RegisterMessage(Query.Set.Values.Strings)
_sym_db.RegisterMessage(Query.Set.Values.Bools)
_sym_db.RegisterMessage(Query.Set.Values.Locations)
_sym_db.RegisterMessage(Query.ConstraintExpr)
_sym_db.RegisterMessage(Query.ConstraintExpr.Or)
_sym_db.RegisterMessage(Query.ConstraintExpr.And)
_sym_db.RegisterMessage(Query.ConstraintExpr.Not)
_sym_db.RegisterMessage(Query.ConstraintExpr.Constraint)
_sym_db.RegisterMessage(Query.Model)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests for the code-blocks in the multiplexer-standalone.md file."""

import logging
import os
from pathlib import Path

from tests.conftest import ROOT_DIR
from tests.test_docs.helper import extract_code_blocks


logger = logging.getLogger(__name__)


class TestDemoDocs:
    """This class contains the tests for the bash/yaml-blocks in *.md file."""

    BASH_DIR_PATH = Path(ROOT_DIR, "tests", "test_docs", "test_bash_yaml", "md_files")

    def _test_blocks(self, filename: str, filter_: str):
        """Test code blocks of a certain type determined by 'filter_' param."""
        bash_file = Path(self.BASH_DIR_PATH, filename).read_text(encoding="utf-8")
        md_path = os.path.join(ROOT_DIR, "docs", filename.replace("bash-", ""))

        bash_code_blocks = extract_code_blocks(filepath=md_path, filter_=filter_)
        for blocks in bash_code_blocks:
            assert blocks in bash_file, "[{}]: FAILED. Code must be identical".format(
                filename
            )
        logger.info(
            f"[{filename}]: PASSED. Tested {len(bash_code_blocks)} '{filter_}' blocks."
        )

    def test_code_blocks_exist(self):
        """Test that all the code-blocks exist in the python file."""
        logger.info(os.listdir(self.BASH_DIR_PATH))
        for file in os.listdir(self.BASH_DIR_PATH):
            if not file.endswith(".md"):
                continue
            # in the future, we might add other filters like "python"
            self._test_blocks(file, "bash")
            self._test_blocks(file, "yaml")
